
/**
  * Returns an array containing all resource codes ordered as defined in the resource files.
  * For example ["food", "wood", "stone", "metal"].
  */
Resources.prototype.GetCodes = function(civ)
{
	if (!civ)
		return this.resourceCodes;

	return this.resourceData.map(res => this.CanUse(civ, res.code) && res.code).filter(x => x);
};

Resources.prototype.CanUse = function(civ, resourceCode)
{
	if (!this.resourceCodes.includes(resourceCode))
		return false;

	let res = this.resourceDataObj[resourceCode];
	return (res.civs || []).includes(civ) || !res.civs && !(res.notcivs || []).includes(civ);
};


/**
 * Part of the ECMA-262 definition, which isn't supported in our version of SpiderMonkey.
 * (Supported from v43 upwards)
 * Thus, very basic polyfill.
 */
if (!("includes" in Array.prototype))
	Object.defineProperty(Array.prototype, 'includes', {
		value: function(value) {
			return this.indexOf(value) != -1;
		}
	});
