function resizeDiplomacyDialog(lastPlayer=null)
{
	if (lastPlayer == g_ViewedPlayer)
		return;

	let dialog = Engine.GetGUIObjectByName("diplomacyDialogPanel");
	let size = dialog.size;

	let tribSize = Engine.GetGUIObjectByName("diplomacyPlayer[0]_tribute[0]").size;
	let iconCount = g_ResourceData.GetCodes((g_Players[g_ViewedPlayer] || { "civ": null }).civ).length;
	if (lastPlayer !== null)
		iconCount = g_ResourceData.GetCodes((g_Players[lastPlayer] || { "civ": null }).civ).length - iconCount;

	let widthOffset = iconCount * (tribSize.right - tribSize.left) / 2;
	size.left -= widthOffset;
	size.right += widthOffset;

	if (lastPlayer !== null)
	{
		dialog.size = size;
		return;
	}

	let firstRow = Engine.GetGUIObjectByName("diplomacyPlayer[0]").size;
	let heightOffset = (g_Players.length - 1) * (firstRow.bottom - firstRow.top) / 2;
	size.top -= heightOffset;
	size.bottom += heightOffset;

	dialog.size = size;
}

function diplomacyFormatTributeButtons(i, hidden)
{
	let resCodes = g_ResourceData.GetCodes(g_Players[g_ViewedPlayer].civ);
	let r = 0;
	for (let resCode of resCodes)
	{
		let button = Engine.GetGUIObjectByName("diplomacyPlayer[" + (i - 1) + "]_tribute[" + r + "]");
		if (!button)
		{
			warn("Current GUI limits prevent displaying more than " + r + " tribute buttons!");
			break;
		}

		Engine.GetGUIObjectByName("diplomacyPlayer[" + (i - 1) + "]_tribute[" + r + "]_image").sprite = "stretched:session/icons/resources/" + resCode + ".png";
		button.hidden = hidden || !g_ResourceData.CanUse(g_Players[i].civ, resCode);
		setPanelObjectPosition(button, r, r + 1, 0);
		++r;
		if (hidden)
			continue;

		button.enabled = controlsPlayer(g_ViewedPlayer);
		button.tooltip = formatTributeTooltip(i, resCode, 100);
		button.onPress = (function(i, resCode, button) {
			// Shift+click to send 500, shift+click+click to send 1000, etc.
			// See INPUT_MASSTRIBUTING in input.js
			let multiplier = 1;
			return function() {
				let isBatchTrainPressed = Engine.HotkeyIsPressed("session.masstribute");
				if (isBatchTrainPressed)
				{
					inputState = INPUT_MASSTRIBUTING;
					multiplier += multiplier == 1 ? 4 : 5;
				}

				let amounts = {};
				for (let res of resCodes)
					amounts[res] = 0;
				amounts[resCode] = 100 * multiplier;

				button.tooltip = formatTributeTooltip(i, resCode, amounts[resCode]);

				// This is in a closure so that we have access to `player`, `amounts`, and `multiplier` without some
				// evil global variable hackery.
				g_FlushTributing = function() {
					Engine.PostNetworkCommand({ "type": "tribute", "player": i, "amounts": amounts });
					multiplier = 1;
					button.tooltip = formatTributeTooltip(i, resCode, 100);
				};

				if (!isBatchTrainPressed)
					g_FlushTributing();
			};
		})(i, resCode, button);
	}
	hideRemaining("diplomacyPlayer[" + (i - 1) + "]_tributes", r);
}

function resizeTradeDialog(lastPlayer=null)
{
	let dialog = Engine.GetGUIObjectByName("tradeDialogPanel");
	let size = dialog.size;
	let width = size.right - size.left;

	let tradeSize = Engine.GetGUIObjectByName("tradeResource[0]").size;
	let iconCount = g_ResourceData.GetCodes((g_Players[g_ViewedPlayer] || { "civ": null }).civ).length;
	if (lastPlayer !== null)
		iconCount = g_ResourceData.GetCodes((g_Players[lastPlayer] || { "civ": null }).civ).length - iconCount;
	width += iconCount * (tradeSize.right - tradeSize.left);

	size.left = -width / 2;
	size.right = width / 2;
	dialog.size = size;
}

function openTrade()
{
	closeOpenDialogs();

	if (g_ViewedPlayer < 1)
		return;

	g_IsTradeOpen = true;

	let proba = Engine.GuiInterfaceCall("GetTradingGoods", g_ViewedPlayer);
	let button = {};
	let resCodes = g_ResourceData.GetCodes(g_Players[g_ViewedPlayer].civ);
	let currTradeSelection = resCodes[0];

	let updateTradeButtons = function()
	{
		for (let res in button)
		{
			button[res].label.caption = proba[res] + "%";

			button[res].sel.hidden = !controlsPlayer(g_ViewedPlayer) || res != currTradeSelection;
			button[res].up.hidden = !controlsPlayer(g_ViewedPlayer) || res == currTradeSelection || proba[res] == 100 || proba[currTradeSelection] == 0;
			button[res].dn.hidden = !controlsPlayer(g_ViewedPlayer) || res == currTradeSelection || proba[res] == 0 || proba[currTradeSelection] == 100;
		}
	};

	hideRemaining("tradeResources", resCodes.length);
	hideRemaining("barterResources", resCodes.length);
	Engine.GetGUIObjectByName("tradeHelp").hidden = false;

	for (let i = 0; i < resCodes.length; ++i)
	{
		let resCode = resCodes[i];

		let barterResource = Engine.GetGUIObjectByName("barterResource[" + i + "]");
		if (!barterResource)
		{
			 warn("Current GUI limits prevent displaying more than " + i + " resources in the barter dialog!");
			 break;
		}
		barterResource.hidden = false;

		// Barter:
		barterOpenCommon(resCode, i, "barter");
		setPanelObjectPosition(barterResource, i, i + 1);

		// Trade:
		let tradeResource = Engine.GetGUIObjectByName("tradeResource[" + i + "]");
		if (!tradeResource)
		{
			 warn("Current GUI limits prevent displaying more than " + i + " resources in the trading goods selection dialog!");
			 break;
		}
		tradeResource.hidden = false;

		setPanelObjectPosition(tradeResource, i, i + 1);

		let icon = Engine.GetGUIObjectByName("tradeResourceIcon[" + i + "]");
		icon.sprite = "stretched:session/icons/resources/" + resCode + ".png";

		let buttonUp = Engine.GetGUIObjectByName("tradeArrowUp[" + i + "]");
		let buttonDn = Engine.GetGUIObjectByName("tradeArrowDn[" + i + "]");

		button[resCode] = {
			"up": buttonUp,
			"dn": buttonDn,
			"label": Engine.GetGUIObjectByName("tradeResourceText[" + i + "]"),
			"sel": Engine.GetGUIObjectByName("tradeResourceSelection[" + i + "]")
		};

		proba[resCode] = proba[resCode] || 0;

		let buttonResource = Engine.GetGUIObjectByName("tradeResourceButton[" + i + "]");
		buttonResource.enabled = controlsPlayer(g_ViewedPlayer);
		buttonResource.onPress = (resource => {
			return () => {
				if (Engine.HotkeyIsPressed("session.fulltradeswap"))
				{
					for (let res of resCodes)
						proba[res] = 0;
					proba[resource] = 100;
					Engine.PostNetworkCommand({ "type": "set-trading-goods", "tradingGoods": proba });
				}
				currTradeSelection = resource;
				updateTradeButtons();
			};
		})(resCode);

		buttonUp.enabled = controlsPlayer(g_ViewedPlayer);
		buttonUp.onPress = (resource => {
			return () => {
				proba[resource] += Math.min(STEP, proba[currTradeSelection]);
				proba[currTradeSelection] -= Math.min(STEP, proba[currTradeSelection]);
				Engine.PostNetworkCommand({ "type": "set-trading-goods", "tradingGoods": proba });
				updateTradeButtons();
			};
		})(resCode);

		buttonDn.enabled = controlsPlayer(g_ViewedPlayer);
		buttonDn.onPress = (resource => {
			return () => {
				proba[currTradeSelection] += Math.min(STEP, proba[resource]);
				proba[resource] -= Math.min(STEP, proba[resource]);
				Engine.PostNetworkCommand({ "type": "set-trading-goods", "tradingGoods": proba });
				updateTradeButtons();
			};
		})(resCode);
	}

	updateTradeButtons();
	updateTraderTexts();

	Engine.GetGUIObjectByName("tradeDialogPanel").hidden = false;
}

function updateBarterButtons()
{
	let playerState = GetSimState().players[g_ViewedPlayer];
	if (!playerState)
		return;

	let canBarter = playerState.canBarter;
	Engine.GetGUIObjectByName("barterNoMarketsMessage").hidden = canBarter;
	Engine.GetGUIObjectByName("barterResources").hidden = !canBarter;
	Engine.GetGUIObjectByName("barterHelp").hidden = !canBarter;

	if (canBarter)
		g_ResourceData.GetCodes(playerState.civ).forEach((resCode, i) => { barterUpdateCommon(resCode, i, "barter", g_ViewedPlayer) });
}
