g_SelectionPanels.Barter.getItems = function(unitEntStates)
{
	let civ = (g_Players[g_ViewedPlayer] || { "civ": null }).civ;
	// If more than `rowLength` resources, don't display icons.
	if (unitEntStates.every(state => !state.isBarterMarket) || g_ResourceData.GetCodes(civ).length > this.rowLength)
		return [];
	return g_ResourceData.GetCodes(civ);
};

g_SelectionPanels.Research.setupButton = function(data)
{
	if (!data.item.tech)
	{
		g_SelectionPanels.Research.hideItem(data.i, data.rowLength);
		return false;
	}

	// Start position (start at the bottom)
	let position = data.i + data.rowLength;

	// Only show the top button for pairs
	if (!data.item.tech.pair)
		Engine.GetGUIObjectByName("unitResearchButton[" + data.i + "]").hidden = true;

	// Set up the tech connector
	let pair = Engine.GetGUIObjectByName("unitResearchPair[" + data.i + "]");
	pair.hidden = data.item.tech.pair == null;
	setPanelObjectPosition(pair, data.i, data.rowLength);

	// Handle one or two techs (tech pair)
	let player = data.player;
	let playerState = GetSimState().players[player];
	for (let tech of data.item.tech.pair ? [data.item.tech.bottom, data.item.tech.top] : [data.item.tech])
	{
		// Don't change the object returned by GetTechnologyData
		let template = clone(GetTechnologyData(tech, playerState.civ));
		if (!template)
			return false;

		for (let res in template.cost)
			template.cost[res] *= data.item.techCostMultiplier[res] || 0;

		let neededResources = Engine.GuiInterfaceCall("GetNeededResources", {
			"cost": template.cost,
			"player": player
		});

		let requirementsPassed = Engine.GuiInterfaceCall("CheckTechnologyRequirements", {
			"tech": tech,
			"player": player
		});

		let button = Engine.GetGUIObjectByName("unitResearchButton[" + position + "]");
		let icon = Engine.GetGUIObjectByName("unitResearchIcon[" + position + "]");

		let tooltips = [
			getEntityNamesFormatted,
			getEntityTooltip,
			getEntityCostTooltip,
			showTemplateViewerOnRightClickTooltip
		].map(func => func(template));

		if (!requirementsPassed)
		{
			let tip = template.requirementsTooltip;
			let reqs = template.reqs;
			for (let req of reqs)
			{
				if (!req.entities)
					continue;

				let entityCounts = [];
				for (let entity of req.entities)
				{
					let current = 0;
					switch (entity.check)
					{
					case "count":
						current = playerState.classCounts[entity.class] || 0;
						break;

					case "variants":
						current = playerState.typeCountsByClass[entity.class] ?
							Object.keys(playerState.typeCountsByClass[entity.class]).length : 0;
						break;
					}

					let remaining = entity.number - current;
					if (remaining < 1)
						continue;

					entityCounts.push(sprintf(translatePlural("%(number)s entity of class %(class)s", "%(number)s entities of class %(class)s", remaining), {
						"number": remaining,
						"class": entity.class
					}));
				}

				tip += " " + sprintf(translate("Remaining: %(entityCounts)s"), {
					"entityCounts": entityCounts.join(translateWithContext("Separator for a list of entity counts", ", "))
				});
			}
			tooltips.push(tip);
		}
		tooltips.push(getNeededResourcesTooltip(neededResources));
		button.tooltip = tooltips.filter(tip => tip).join("\n");

		button.onPress = (t => function() {
			addResearchToQueue(data.item.researchFacilityId, t);
		})(tech);

		button.onPressRight = (t => function () {
			showTemplateDetails(
				t,
				GetTemplateData(data.unitEntStates.find(state => state.id == data.item.researchFacilityId).template).nativeCiv);
		})(tech);

		if (data.item.tech.pair)
		{
			// On mouse enter, show a cross over the other icon
			let unchosenIcon = Engine.GetGUIObjectByName("unitResearchUnchosenIcon[" + (position + data.rowLength) % (2 * data.rowLength) + "]");
			button.onMouseEnter = function() {
				unchosenIcon.hidden = false;
			};
			button.onMouseLeave = function() {
				unchosenIcon.hidden = true;
			};
		}

		button.hidden = false;
		let modifier = "";
		if (!requirementsPassed)
		{
			button.enabled = false;
			modifier += "color:0 0 0 127:grayscale:";
		}
		else if (neededResources)
		{
			button.enabled = false;
			modifier += resourcesToAlphaMask(neededResources) + ":";
		}
		else
			button.enabled = controlsPlayer(data.player);

		if (template.icon)
			icon.sprite = modifier + "stretched:session/portraits/" + template.icon;

		setPanelObjectPosition(button, position, data.rowLength);

		// Prepare to handle the top button (if any)
		position -= data.rowLength;
	}

	return true;
}
